@@ -187,12 +187,12 @@ combine_hlist <- function(hlist, class, alpha) {
   # combine list of outputs of eval_hypothesis
   # Returns: a brmshypothesis object
   stopifnot(is.list(hlist))
-  hs <- run(rbind, lapply(hlist, function(h) h$summary))
+  hs <- do_call(rbind, lapply(hlist, function(h) h$summary))
   rownames(hs) <- NULL
   samples <- lapply(hlist, function(h) h$samples)
-  samples <- as.data.frame(run(cbind, samples))
+  samples <- as.data.frame(do_call(cbind, samples))
   prior_samples <- lapply(hlist, function(h) h$prior_samples)
-  prior_samples <- as.data.frame(run(cbind, prior_samples))
+  prior_samples <- as.data.frame(do_call(cbind, prior_samples))
   names(samples) <- names(prior_samples) <- paste0("H", seq_along(hlist))
   class <- sub("_+$", "", class)
   out <- nlist(hypothesis = hs, samples, prior_samples, class, alpha)
@@ -295,7 +295,9 @@ find_vars <- function(x, dot = TRUE, brackets = TRUE) {
   )
   pos_fun <- gregexpr(regex_fun, x)[[1]]
   pos_decnum <- gregexpr("\\.[[:digit:]]+", x)[[1]]
-  pos_var <- rmMatch(pos_all, pos_fun, pos_decnum)
+  keep <- !pos_all %in% c(pos_fun, pos_decnum)
+  pos_var <- pos_all[keep]
+  attr(pos_var, "match.length") <- attributes(pos_all)$match.length[keep]
   if (length(pos_var)) {
     out <- unique(unlist(regmatches(x, list(pos_var))))
   } else {
@@ -350,7 +352,7 @@ density_ratio <- function(x, y = NULL, point = 0, n = 4096, ...) {
     } else if (to < point) {
       to <- point + sd(x) / 4
     }
-    dens <- run(density, c(nlist(x, from, to), dots))
+    dens <- do_call(density, c(nlist(x, from, to), dots))
     return(spline(dens$x, dens$y, xout = point)$y)
   }
   
