@@ -1,5 +1,5 @@
 params <-
-list(EVAL = FALSE)
+list(EVAL = TRUE)
 
 ## ---- SETTINGS-knitr, include=FALSE-----------------------------------------------------
 stopifnot(require(knitr))
@@ -17,83 +17,83 @@ opts_chunk$set(
   fig.align = "center"
 )
 library(brms)
-theme_set(theme_default())
+ggplot2::theme_set(theme_default())
 
 ## ----cbpp-------------------------------------------------------------------------------
-#  data("cbpp", package = "lme4")
-#  head(cbpp)
+data("cbpp", package = "lme4")
+head(cbpp)
 
 ## ----fit1, results='hide'---------------------------------------------------------------
-#  fit1 <- brm(incidence | trials(size) ~ period + (1|herd),
-#              data = cbpp, family = binomial())
+fit1 <- brm(incidence | trials(size) ~ period + (1|herd), 
+            data = cbpp, family = binomial())
 
 ## ----fit1_summary-----------------------------------------------------------------------
-#  summary(fit1)
+summary(fit1)
 
 ## ----beta_binomial2---------------------------------------------------------------------
-#  beta_binomial2 <- custom_family(
-#    "beta_binomial2", dpars = c("mu", "phi"),
-#    links = c("logit", "log"), lb = c(NA, 0),
-#    type = "int", vars = "trials[n]"
-#  )
+beta_binomial2 <- custom_family(
+  "beta_binomial2", dpars = c("mu", "phi"),
+  links = c("logit", "log"), lb = c(NA, 0),
+  type = "int", vars = "trials[n]"
+)
 
 ## ----stan_funs--------------------------------------------------------------------------
-#  stan_funs <- "
-#    real beta_binomial2_lpmf(int y, real mu, real phi, int T) {
-#      return beta_binomial_lpmf(y | T, mu * phi, (1 - mu) * phi);
-#    }
-#    int beta_binomial2_rng(real mu, real phi, int T) {
-#      return beta_binomial_rng(T, mu * phi, (1 - mu) * phi);
-#    }
-#  "
+stan_funs <- "
+  real beta_binomial2_lpmf(int y, real mu, real phi, int T) {
+    return beta_binomial_lpmf(y | T, mu * phi, (1 - mu) * phi);
+  }
+  int beta_binomial2_rng(real mu, real phi, int T) {
+    return beta_binomial_rng(T, mu * phi, (1 - mu) * phi);
+  }
+"
 
 ## ----stanvars---------------------------------------------------------------------------
-#  stanvars <- stanvar(scode = stan_funs, block = "functions") +
-#    stanvar(as.integer(cbpp$size), name = "trials")
+stanvars <- stanvar(scode = stan_funs, block = "functions") +
+  stanvar(as.integer(cbpp$size), name = "trials")
 
 ## ----fit2, results='hide'---------------------------------------------------------------
-#  fit2 <- brm(
-#    incidence ~ period + (1|herd), data = cbpp,
-#    family = beta_binomial2, stanvars = stanvars
-#  )
+fit2 <- brm(
+  incidence ~ period + (1|herd), data = cbpp, 
+  family = beta_binomial2, stanvars = stanvars
+)
 
 ## ----summary_fit2-----------------------------------------------------------------------
-#  summary(fit2)
+summary(fit2)
 
 ## ---------------------------------------------------------------------------------------
-#  expose_functions(fit2, vectorize = TRUE)
+expose_functions(fit2, vectorize = TRUE)
 
 ## ----log_lik----------------------------------------------------------------------------
-#  log_lik_beta_binomial2 <- function(i, draws) {
-#    mu <- draws$dpars$mu[, i]
-#    phi <- draws$dpars$phi
-#    N <- draws$data$trials[i]
-#    y <- draws$data$Y[i]
-#    beta_binomial2_lpmf(y, mu, phi, N)
-#  }
+log_lik_beta_binomial2 <- function(i, draws) {
+  mu <- draws$dpars$mu[, i]
+  phi <- draws$dpars$phi
+  N <- draws$data$trials[i]
+  y <- draws$data$Y[i]
+  beta_binomial2_lpmf(y, mu, phi, N)
+}
 
 ## ----loo--------------------------------------------------------------------------------
-#  loo(fit1, fit2)
+loo(fit1, fit2)
 
 ## ----predict----------------------------------------------------------------------------
-#  predict_beta_binomial2 <- function(i, draws, ...) {
-#    mu <- draws$dpars$mu[, i]
-#    phi <- draws$dpars$phi
-#    N <- draws$data$trials[i]
-#    beta_binomial2_rng(mu, phi, N)
-#  }
+predict_beta_binomial2 <- function(i, draws, ...) {
+  mu <- draws$dpars$mu[, i]
+  phi <- draws$dpars$phi
+  N <- draws$data$trials[i]
+  beta_binomial2_rng(mu, phi, N)
+}
 
 ## ----pp_check---------------------------------------------------------------------------
-#  pp_check(fit2)
+pp_check(fit2)
 
 ## ----fitted-----------------------------------------------------------------------------
-#  fitted_beta_binomial2 <- function(draws) {
-#    mu <- draws$dpars$mu
-#    trials <- draws$data$trials
-#    trials <- matrix(trials, nrow = nrow(mu), ncol = ncol(mu), byrow = TRUE)
-#    mu * trials
-#  }
+fitted_beta_binomial2 <- function(draws) {
+  mu <- draws$dpars$mu
+  trials <- draws$data$trials
+  trials <- matrix(trials, nrow = nrow(mu), ncol = ncol(mu), byrow = TRUE)
+  mu * trials
+}
 
 ## ----marginal_effects-------------------------------------------------------------------
-#  marginal_effects(fit2, new_objects = list(trials = 1))
+marginal_effects(fit2, new_objects = list(trials = 1))
 
