@@ -35,7 +35,7 @@ test_that("specified priors appear in the Stan code", {
              prior(normal(0, 5), class = Intercept),
              prior(cauchy(0, 5), class = sd))
   scode <- make_stancode(y ~ x1 + cs(x2) + (0 + x1 + x2 | g), 
-                         dat, family = acat(), 
+                         data = dat, family = acat(), 
                          prior = prior, sample_prior = TRUE)
   expect_match2(scode, "target += normal_lpdf(b | 0, 1)")
   expect_match2(scode, "target += normal_lpdf(temp_Intercept | 0, 5)")
@@ -63,7 +63,7 @@ test_that("specified priors appear in the Stan code", {
   prior <- c(prior(lkj(2), rescor),
              prior(cauchy(0, 5), sigma, resp = y),
              prior(cauchy(0, 1), sigma, resp = x1))
-  scode <- make_stancode(cbind(y, x1) ~ x2, dat, prior = prior, 
+  scode <- make_stancode(mvbind(y, x1) ~ x2, dat, prior = prior, 
                          sample_prior = TRUE)
   expect_match2(scode, "target += lkj_corr_cholesky_lpdf(Lrescor | 2)")
   expect_match2(scode, "prior_sigma_y = cauchy_rng(0,5)")
@@ -225,7 +225,7 @@ test_that("special shrinkage priors appear in the Stan code", {
                "Scale of the global prior")
   expect_error(make_stancode(y ~ x1*x2, data = dat, prior = prior(lasso(-1))),
                "Degrees of freedom of the shrinkage parameter prior")
-  expect_error(make_stancode(x1 ~ cs(y), dat, acat(), prior = prior(lasso())),
+  expect_error(make_stancode(y ~ cs(x1), dat, acat(), prior = prior(lasso())),
                "Horseshoe or lasso priors are not yet allowed")
   bprior <- prior(horseshoe()) + prior(normal(0, 1), coef = "y")
   expect_error(make_stancode(x1 ~ y, dat, prior = bprior),
@@ -238,7 +238,7 @@ test_that("link functions appear in the Stan code", {
   dat <- data.frame(y = 1:10, x = rnorm(10))
   expect_match2(make_stancode(y ~ x, dat, family = poisson()), 
                "target += poisson_log_lpmf(Y | mu);")
-  expect_match2(make_stancode(cbind(y, y + 1) ~ x, dat, family = gaussian("log")), 
+  expect_match2(make_stancode(mvbind(y, y + 1) ~ x, dat, family = gaussian("log")), 
                "mu_y[n] = exp(mu_y[n]);")
   expect_match2(make_stancode(y ~ x, dat, family = von_mises(tan_half)), 
                "mu[n] = inv_tan_half(mu[n]);")
@@ -255,17 +255,19 @@ test_that("link functions appear in the Stan code", {
 test_that("customized covariances appear in the Stan code", {
   scode <- make_stancode(rating ~ treat + period + carry + (1|subject), 
                          data = inhaler, cov_ranef = list(subject = 1))
-  expect_match2(scode, "r_1_1 = sd_1[1] * (Lcov_1 * z_1[1])")
+  expect_match2(scode, "r_1_1 = (sd_1[1] * (Lcov_1 * z_1[1]))")
   
   scode <- make_stancode(rating ~ treat + period + carry + (1+carry|subject), 
                          data = inhaler, cov_ranef = list(subject = 1))
   expect_match2(scode,
-    "kronecker(Lcov_1, diag_pre_multiply(sd_1, L_1)) * to_vector(z_1)")
+    "kronecker(Lcov_1, diag_pre_multiply(sd_1, L_1)) * to_vector(z_1)"
+  )
+  expect_match2(scode, "cor_1[choose(k - 1, 2) + j] = Cor_1[j, k];")
   
   scode <- make_stancode(rating ~ treat + period + carry + (1+carry||subject), 
                          data = inhaler, cov_ranef = list(subject = 1))
-  expect_match2(scode, " r_1_1 = sd_1[1] * (Lcov_1 * z_1[1]);")
-  expect_match2(scode, " r_1_2 = sd_1[2] * (Lcov_1 * z_1[2]);")
+  expect_match2(scode, " r_1_1 = (sd_1[1] * (Lcov_1 * z_1[1]));")
+  expect_match2(scode, " r_1_2 = (sd_1[2] * (Lcov_1 * z_1[2]));")
 })
 
 test_that("truncation appears in the Stan code", {
@@ -280,27 +282,27 @@ test_that("truncation appears in the Stan code", {
   expect_match2(scode, "target += student_t_lpdf(Y[n] | nu, mu[n], sigma) -")
   expect_match2(scode, "student_t_lcdf(ub[n] | nu, mu[n], sigma);")
   
-  scode <- make_stancode(count | trunc(0, 150) ~ Trt_c, 
+  scode <- make_stancode(count | trunc(0, 150) ~ Trt, 
                          data = epilepsy, family = "poisson")
   expect_match2(scode, "target += poisson_lpmf(Y[n] | mu[n]) -")
   expect_match2(scode, 
-    "log_diff_exp(poisson_lcdf(ub[n] | mu[n]), poisson_lcdf(lb[n] | mu[n]));"
+    "log_diff_exp(poisson_lcdf(ub[n] | mu[n]), poisson_lcdf(lb[n] - 1 | mu[n]));"
   )
 })
 
 test_that("make_stancode combines strings of multiple grouping factors", {
-  expect_match2(make_stancode(count ~ (1|patient) + (1 + Trt_c | visit), 
+  expect_match2(make_stancode(count ~ (1|patient) + (1 + Trt | visit), 
                              data = epilepsy, family = "poisson"), 
                paste0("  vector[N] Z_1_1;\n",
                       "  // data for group-level effects of ID 2"))
-  expect_match2(make_stancode(count ~ (1|visit) + (1+Trt_c|patient), 
+  expect_match2(make_stancode(count ~ (1|visit) + (1+Trt|patient), 
                              data = epilepsy, family = skew_normal()), 
                paste0("  int<lower=1> NC_1;\n",
                       "  // data for group-level effects of ID 2"))
 })
 
 test_that("make_stancode handles models without fixed effects", {
-  expect_match2(make_stancode(count ~ 0 + (1|patient) + (1+Trt_c|visit), 
+  expect_match2(make_stancode(count ~ 0 + (1|patient) + (1+Trt|visit), 
                              data = epilepsy, family = "poisson"), 
                "mu = rep_vector(0, N);")
 })
@@ -352,31 +354,31 @@ test_that("self-defined functions appear in the Stan code", {
   expect_match2(scode, "real von_mises_vector_lpdf(vector y")
   
   # zero-inflated and hurdle models
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                               family = "zero_inflated_poisson"),
                "real zero_inflated_poisson_lpmf(int y")
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                              family = "zero_inflated_negbinomial"),
                "real zero_inflated_neg_binomial_lpmf(int y")
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                               family = "zero_inflated_binomial"),
                "real zero_inflated_binomial_lpmf(int y")
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                              family = "zero_inflated_beta"),
                "real zero_inflated_beta_lpdf(real y")
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                               family = "zero_one_inflated_beta"),
                 "real zero_one_inflated_beta_lpdf(real y")
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                              family = hurdle_poisson()),
                "real hurdle_poisson_lpmf(int y")
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                              family = hurdle_negbinomial),
                "real hurdle_neg_binomial_lpmf(int y")
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                              family = hurdle_gamma("log")),
                "real hurdle_gamma_lpdf(real y")
-  expect_match2(make_stancode(count ~ Trt_c, data = epilepsy, 
+  expect_match2(make_stancode(count ~ Trt, data = epilepsy, 
                              family = hurdle_lognormal("identity")),
                "real hurdle_lognormal_lpdf(real y")
   
@@ -413,7 +415,7 @@ test_that("invalid combinations of modeling options are detected", {
     "Invalid addition arguments for this model"
   )
   expect_error(
-    make_stancode(cbind(y1, y2) ~ 1, data = data, autocor = cor_ar(cov = TRUE)),
+    make_stancode(mvbind(y1, y2) ~ 1, data = data, autocor = cor_ar(cov = TRUE)),
     "ARMA covariance matrices are not implemented when 'rescor' is estimated."
   )
   expect_error(
@@ -429,15 +431,16 @@ test_that("Stan code for multivariate models is correct", {
     censi = sample(0:1, 10, TRUE)
   )
   # models with residual correlations
-  scode <- make_stancode(cbind(y1, y2) ~ x, dat, prior = prior(horseshoe(2)))
+  scode <- make_stancode(mvbind(y1, y2) ~ x, dat, prior = prior(horseshoe(2)))
   expect_match2(scode, "target += multi_normal_cholesky_lpdf(Y | Mu, LSigma);")
   expect_match2(scode, "LSigma = diag_pre_multiply(sigma, Lrescor);")
   expect_match2(scode, "target += normal_lpdf(hs_local_y1[1] | 0, 1)")
   expect_match2(scode, 
     "target += inv_gamma_lpdf(hs_local_y2[2] | 0.5 * hs_df_y2, 0.5 * hs_df_y2)"
   )
+  expect_match2(scode, "rescor[choose(k - 1, 2) + j] = Rescor[j, k];")
   
-  scode <- make_stancode(cbind(y1, y2) ~ x, dat, student(),
+  scode <- make_stancode(mvbind(y1, y2) ~ x, dat, student(),
                          prior = prior(lasso(2, 10)))
   expect_match2(scode, "target += multi_student_t_lpdf(Y | nu, Mu, Sigma);")
   expect_match2(scode, "matrix[nresp, nresp] Sigma = multiply_lower")
@@ -448,7 +451,7 @@ test_that("Stan code for multivariate models is correct", {
   expect_match2(scode, 
     "target += chi_square_lpdf(lasso_inv_lambda_y2 | lasso_df_y2)"
   )
-  expect_match2(make_stancode(cbind(y1, y2) | weights(x) ~ 1, dat),
+  expect_match2(make_stancode(mvbind(y1, y2) | weights(x) ~ 1, dat),
     "target += weights[n] * multi_normal_cholesky_lpdf(Y[n] | Mu[n], LSigma);"
   )
   
@@ -481,21 +484,69 @@ test_that("Stan code for multivariate models is correct", {
 
 test_that("Stan code for categorical models is correct", {
   dat <- data.frame(y = rep(1:4, 2), x = 1:8, g = 1:8)
-  prior <- c(
-    prior(normal(0, 5), "b"),
-    prior(normal(0, 10), "b", dpar = mu2),
-    prior(cauchy(0, 1), "Intercept"),
+  prior <- prior(normal(0, 5), "b") +
+    prior(normal(0, 10), "b", dpar = mu2) +
+    prior(cauchy(0, 1), "Intercept") +
     prior(normal(0, 2), "Intercept", dpar = mu3)
-  )
+  
   scode <- make_stancode(y ~ x + (1|ID|g), data = dat, 
                          family = categorical(), prior = prior)
   expect_match2(scode, "target += categorical_logit_lpmf(Y[n] | mu[n]);")
+  expect_match2(scode, "mu[n] = [0, mu2[n], mu3[n], mu4[n]]';")
   expect_match2(scode, "mu2 = temp_mu2_Intercept + Xc_mu2 * b_mu2;")
   expect_match2(scode, "mu4[n] += r_1_mu4_3[J_1[n]] * Z_1_mu4_3[n];")
   expect_match2(scode, "target += normal_lpdf(b_mu2 | 0, 10);")
   expect_match2(scode, "target += normal_lpdf(b_mu4 | 0, 5);")
   expect_match2(scode, "target += cauchy_lpdf(temp_mu2_Intercept | 0, 1);")
   expect_match2(scode, "target += normal_lpdf(temp_mu3_Intercept | 0, 2);")
+  
+  scode <- make_stancode(y ~ x + (1|ID|g), data = dat, 
+                         family = categorical(refcat = NA))
+  expect_match2(scode, "mu[n] = [mu1[n], mu2[n], mu3[n], mu4[n]]';")
+})
+
+test_that("Stan code for multinomial models is correct", {
+  N <- 15
+  dat <- data.frame(
+    y1 = rbinom(N, 10, 0.3), y2 = rbinom(N, 10, 0.5), 
+    y3 = rbinom(N, 10, 0.7), x = rnorm(N)
+  )
+  dat$size <- with(dat, y1 + y2 + y3)
+  dat$y <- with(dat, cbind(y1, y2, y3))
+  prior <- prior(normal(0, 10), "b", dpar = muy2) +
+    prior(cauchy(0, 1), "Intercept") +
+    prior(normal(0, 2), "Intercept", dpar = muy3)
+  scode <- make_stancode(bf(y | trials(size)  ~ 1, muy2 ~ x), data = dat, 
+                         family = multinomial(), prior = prior)
+  expect_match2(scode, "int Y[N, ncat];")
+  expect_match2(scode, "target += multinomial_logit_lpmf(Y[n] | mu[n]);")
+  expect_match2(scode, "muy2 = temp_muy2_Intercept + Xc_muy2 * b_muy2;")
+  expect_match2(scode, "target += normal_lpdf(b_muy2 | 0, 10);")
+  expect_match2(scode, "target += cauchy_lpdf(temp_muy2_Intercept | 0, 1);")
+  expect_match2(scode, "target += normal_lpdf(temp_muy3_Intercept | 0, 2);")
+})
+
+test_that("Stan code for dirichlet models is correct", {
+  N <- 15
+  dat <- as.data.frame(rdirichlet(N, c(3, 2, 1)))
+  names(dat) <- c("y1", "y2", "y3")
+  dat$x <- rnorm(N)
+  dat$y <- with(dat, cbind(y1, y2, y3))
+  prior <- prior(normal(0, 5), "b") +
+    prior(exponential(10), "phi")
+  scode <- make_stancode(bf(y ~ 1, muy3 ~ x), data = dat, 
+                         family = dirichlet(), prior = prior)
+  expect_match2(scode, "vector[ncat] Y[N];")
+  expect_match2(scode, "target += dirichlet_logit_lpdf(Y[n] | mu[n], phi);")
+  expect_match2(scode, "muy3 = temp_muy3_Intercept + Xc_muy3 * b_muy3;")
+  expect_match2(scode, "target += normal_lpdf(b_muy3 | 0, 5);")
+  expect_match2(scode, "target += exponential_lpdf(phi | 10);")
+  
+  scode <- make_stancode(bf(y ~ x, phi ~ x), data = dat, 
+                         family = dirichlet())
+  expect_match2(scode, "target += dirichlet_logit_lpdf(Y[n] | mu[n], phi[n]);")
+  expect_match2(scode, "vector[N] phi = temp_phi_Intercept + Xc_phi * b_phi;")
+  expect_match2(scode, "phi[n] = exp(phi[n]);")
 })
 
 test_that("Stan code for ARMA models is correct", {
@@ -515,7 +566,7 @@ test_that("Stan code for ARMA models is correct", {
   )
   expect_match2(scode, "mu = temp_Intercept + Xc * b + Yarr * arr;")
   
-  scode <- make_stancode(cbind(y, x) ~ 1, dat, gaussian(),
+  scode <- make_stancode(mvbind(y, x) ~ 1, dat, gaussian(),
                          autocor = cor_ar())
   expect_match2(scode, "e_y[n] = Y_y[n] - mu_y[n];")
 })
@@ -732,9 +783,9 @@ test_that("known standard errors appear in the Stan code", {
 })
 
 test_that("functions defined in 'stan_funs' appear in the functions block", {
-  test_fun <- paste0("  real test_fun(real a, real b) { \n",
-                     "    return a + b; \n",
-                     "  } \n")
+  test_fun <- paste0("  real test_fun(real a, real b) {\n",
+                     "    return a + b;\n",
+                     "  }\n")
   scode <- SW(make_stancode(time ~ age, data = kidney, stan_funs = test_fun))
   expect_match2(scode, test_fun)
 })
@@ -804,7 +855,7 @@ test_that("Stan code for GAMMs is correct", {
 test_that("Stan code of response times models is correct", {
   dat <- epilepsy
   dat$cens <- sample(-1:1, nrow(dat), TRUE)
-  scode <- make_stancode(count ~ Trt_c + (1|patient),
+  scode <- make_stancode(count ~ Trt + (1|patient),
                       data = dat, family = exgaussian("log"),
                       prior = prior(gamma(1,1), class = beta))
   expect_match2(scode,
@@ -813,22 +864,22 @@ test_that("Stan code of response times models is correct", {
   expect_match2(scode, "mu[n] = exp(mu[n])")
   expect_match2(scode, "target += gamma_lpdf(beta | 1, 1)")
   
-  scode <- make_stancode(bf(count ~ Trt_c + (1|patient),
-                         sigma ~ Trt_c, beta ~ Trt_c),
+  scode <- make_stancode(bf(count ~ Trt + (1|patient),
+                         sigma ~ Trt, beta ~ Trt),
                       data = dat, family = exgaussian())
   expect_match2(scode, 
     "target += exp_mod_normal_lpdf(Y | mu - beta, sigma, inv(beta))"
   )
   expect_match2(scode, "beta[n] = exp(beta[n])")
   
-  scode <- make_stancode(count | cens(cens) ~ Trt_c + (1|patient),
+  scode <- make_stancode(count | cens(cens) ~ Trt + (1|patient),
                       data = dat, family = exgaussian("inverse"))
   expect_match2(scode, "exp_mod_normal_lccdf(Y[n] | mu[n] - beta, sigma, inv(beta))")
   
-  scode <- make_stancode(count ~ Trt_c, dat, family = shifted_lognormal())
+  scode <- make_stancode(count ~ Trt, dat, family = shifted_lognormal())
   expect_match2(scode, "target += lognormal_lpdf(Y - ndt | mu, sigma)")
   
-  scode <- make_stancode(count | cens(cens) ~ Trt_c, dat, family = shifted_lognormal())
+  scode <- make_stancode(count | cens(cens) ~ Trt, dat, family = shifted_lognormal())
   expect_match2(scode, "target += lognormal_lcdf(Y[n] - ndt | mu[n], sigma)")
 })
 
@@ -846,23 +897,27 @@ test_that("Stan code of wiener diffusion models is correct", {
   )
   expect_match2(scode, "bias[n] = inv_logit(bias[n]);")
   
+  scode <- make_stancode(bf(q | dec(resp) ~ x, ndt = 0.5), 
+                         data = dat, family = wiener())
+  expect_match2(scode, "real<lower=0,upper=min(Y)> ndt;")
+  
   expect_error(make_stancode(q ~ x, data = dat, family = wiener()),
                "Addition argument 'dec' is required for family 'wiener'")
 })
 
 test_that("Group IDs appear in the Stan code", {
-  form <- bf(count ~ Trt_c + (1+Trt_c|3|visit) + (1|patient), 
-             shape ~ (1|3|visit) + (Trt_c||patient))
+  form <- bf(count ~ Trt + (1+Trt|3|visit) + (1|patient), 
+             shape ~ (1|3|visit) + (Trt||patient))
   scode <- make_stancode(form, data = epilepsy, family = negbinomial())
   expect_match2(scode, "r_2_1 = r_2[, 1]")
   expect_match2(scode, "r_2_shape_3 = r_2[, 3]")
   
-  form <- bf(count ~ a, sigma ~ (1|3|visit) + (Trt_c||patient),
-             a ~ Trt_c + (1+Trt_c|3|visit) + (1|patient), nl = TRUE)
+  form <- bf(count ~ a, sigma ~ (1|3|visit) + (Trt||patient),
+             a ~ Trt + (1+Trt|3|visit) + (1|patient), nl = TRUE)
   scode <- make_stancode(form, data = epilepsy, family = student(),
                       prior = set_prior("normal(0,5)", nlpar = "a"))
   expect_match2(scode, "r_2_a_2 = r_2[, 2];")
-  expect_match2(scode, "r_1_sigma_2 = sd_1[2] * (z_1[2]);")
+  expect_match2(scode, "r_1_sigma_2 = (sd_1[2] * (z_1[2]));")
 })
 
 test_that("distributional gamma models are handled correctly", {
@@ -873,7 +928,7 @@ test_that("distributional gamma models are handled correctly", {
     data = kidney, family = Gamma("log")
   )
   expect_match2(scode, paste0(
-    "    shape[n] = exp(shape[n]); \n", 
+    "    shape[n] = exp(shape[n]);\n", 
     "    mu[n] = shape[n] * exp(-(mu[n]));"))
   
   scode <- make_stancode(
@@ -885,8 +940,8 @@ test_that("distributional gamma models are handled correctly", {
               set_prior("normal(0,3)", nlpar = "b"))
   )
   expect_match2(scode, paste0(
-    "    shape[n] = exp(shape[n]); \n", 
-    "    // compute non-linear predictor \n",
+    "    shape[n] = exp(shape[n]);\n", 
+    "    // compute non-linear predictor\n",
     "    mu[n] = shape[n] / (inv_logit(nlp_a[n]) * exp(nlp_b[n] * C_1[n]));"
   ))
   
@@ -897,9 +952,9 @@ test_that("distributional gamma models are handled correctly", {
   )
   # test that no link function is applied on 'shape'
   expect_match2(scode, paste0(
-    "  for (n in 1:N) { \n",
-    "    mu[n] = shape[n] * exp(-(mu[n])); \n",
-    "  } \n"
+    "  for (n in 1:N) {\n",
+    "    mu[n] = shape[n] * exp(-(mu[n]));\n",
+    "  }\n"
   ))
 })
 
@@ -967,6 +1022,7 @@ test_that("noise-free terms appear in the Stan code", {
   expect_match2(scode, "target += cauchy_lpdf(sdme_1[2] | 0, 5)")
   expect_match2(scode, "target += lkj_corr_cholesky_lpdf(Lme_1 | 2)")
   expect_match2(scode, "+ (diag_pre_multiply(sdme_1, Lme_1) * zme_1)'")
+  expect_match2(scode, "corme_1[choose(k - 1, 2) + j] = Corme_1[j, k];")
   
   scode <- make_stancode(
     y ~ me(x, xsd)*z + (me(x, xsd)*z|ID), data = dat
@@ -994,8 +1050,8 @@ test_that("noise-free terms appear in the Stan code", {
   expect_match2(scode, "nlp_a[n] += (bsp_a[1]) * Xme_1[n]")
   expect_match2(scode, "nlp_b[n] += (bsp_b[1]) * Xme_1[n]")
   
-  bform <- bf(cbind(y, z) ~ me(x, xsd)) + set_mecor(FALSE)
-  scode <- make_stancode(cbind(y, z) ~ me(x, xsd), dat)
+  bform <- bf(mvbind(y, z) ~ me(x, xsd)) + set_mecor(FALSE)
+  scode <- make_stancode(mvbind(y, z) ~ me(x, xsd), dat)
   expect_match2(scode, "mu_y[n] += (bsp_y[1]) * Xme_1[n]")
   expect_match2(scode, "mu_z[n] += (bsp_z[1]) * Xme_1[n]")
   expect_match2(scode, "vector[N] Xme_1 = meanme_1[1] + sdme_1[1] * zme_1;")
@@ -1035,7 +1091,7 @@ test_that("by variables in grouping terms are handled correctly", {
     z = factor(rep(c(0, 4.5, 3, 2, 5), each = 20))
   )
   scode <- make_stancode(y ~ x + (1 | gr(g, by = z)), dat)
-  expect_match2(scode, "r_1_1 = sd_1[1, Jby_1]' .* (z_1[1]);")
+  expect_match2(scode, "r_1_1 = (sd_1[1, Jby_1]' .* (z_1[1]));")
   scode <- make_stancode(y ~ x + (x | gr(g, by = z)), dat)
   expect_match2(scode, "r_1 = scale_r_cor_by(z_1, sd_1, L_1, Jby_1);")
   expect_match2(scode, "target += student_t_lpdf(to_vector(sd_1) | 3, 0, 10);")
@@ -1046,13 +1102,13 @@ test_that("Group syntax | and || is handled correctly,", {
   data <- data.frame(y = rnorm(10), x = rnorm(10),
                      g1 = rep(1:5, each = 2), g2 = rep(1:2, 5))
   scode <- make_stancode(y ~ x + (1+x||g1) + (I(x/4)|g2), data)
-  expect_match2(scode, "r_1_2 = sd_1[2] * (z_1[2]);")
+  expect_match2(scode, "r_1_2 = (sd_1[2] * (z_1[2]));")
   expect_match2(scode, "r_2_1 = r_2[, 1];")
   expect_match2(scode, "r_2 = (diag_pre_multiply(sd_2, L_2) * z_2)';")
 })
 
 test_that("predicting zi and hu works correctly", {
-  scode <- make_stancode(bf(count ~ Trt_c, zi ~ Trt_c), epilepsy, 
+  scode <- make_stancode(bf(count ~ Trt, zi ~ Trt), epilepsy, 
                          family = "zero_inflated_poisson")
   expect_match2(scode, 
     "target += zero_inflated_poisson_log_logit_lpmf(Y[n] | mu[n], zi[n])"
@@ -1060,13 +1116,13 @@ test_that("predicting zi and hu works correctly", {
   expect_true(!grepl("inv_logit\\(", scode))
   expect_true(!grepl("exp(mu[n])", scode, fixed = TRUE))
   
-  scode <- make_stancode(bf(count ~ Trt_c, zi ~ Trt_c), epilepsy, 
+  scode <- make_stancode(bf(count ~ Trt, zi ~ Trt), epilepsy, 
                          family = zero_inflated_poisson(identity))
   expect_match2(scode, 
     "target += zero_inflated_poisson_logit_lpmf(Y[n] | mu[n], zi[n])"
   )
   
-  scode <- make_stancode(bf(count ~ Trt_c, zi ~ Trt_c), epilepsy, 
+  scode <- make_stancode(bf(count ~ Trt, zi ~ Trt), epilepsy, 
                          family = "zero_inflated_binomial")
   expect_match2(scode, 
     "target += zero_inflated_binomial_blogit_logit_lpmf(Y[n] | trials[n], mu[n], zi[n])"
@@ -1074,7 +1130,7 @@ test_that("predicting zi and hu works correctly", {
   expect_true(!grepl("inv_logit\\(", scode))
   
   fam <- zero_inflated_binomial("probit", link_zi = "identity")
-  scode <- make_stancode(bf(count ~ Trt_c, zi ~ Trt_c), epilepsy, 
+  scode <- make_stancode(bf(count ~ Trt, zi ~ Trt), epilepsy, 
                          family = fam)
   expect_match2(scode, 
     "target += zero_inflated_binomial_lpmf(Y[n] | trials[n], mu[n], zi[n])"
@@ -1082,14 +1138,14 @@ test_that("predicting zi and hu works correctly", {
   expect_match2(scode, "mu[n] = Phi(mu[n]);")
   
   scode <- make_stancode(
-    bf(count ~ Trt_c, zi ~ Trt_c), epilepsy, 
+    bf(count ~ Trt, zi ~ Trt), epilepsy, 
     family = zero_inflated_beta()
   )
   expect_match2(scode,
     "target += zero_inflated_beta_logit_lpdf(Y[n] | mu[n], phi, zi[n])"      
   )
   
-  scode <- make_stancode(bf(count ~ Trt_c, hu ~ Trt_c), epilepsy, 
+  scode <- make_stancode(bf(count ~ Trt, hu ~ Trt), epilepsy, 
                          family = "hurdle_negbinomial")
   expect_match2(scode, 
     "target += hurdle_neg_binomial_log_logit_lpmf(Y[n] | mu[n], shape, hu[n])"
@@ -1097,7 +1153,7 @@ test_that("predicting zi and hu works correctly", {
   expect_true(!grepl("inv_logit\\(", scode))
   expect_true(!grepl("exp(mu[n])", scode, fixed = TRUE))
   
-  scode <- make_stancode(bf(count ~ Trt_c, hu ~ Trt_c), epilepsy, 
+  scode <- make_stancode(bf(count ~ Trt, hu ~ Trt), epilepsy, 
                          family = "hurdle_gamma")
   expect_match2(scode, 
     "target += hurdle_gamma_logit_lpdf(Y[n] | shape, mu[n], hu[n])"
@@ -1106,7 +1162,7 @@ test_that("predicting zi and hu works correctly", {
   expect_match2(scode, "mu[n] = shape * exp(-(mu[n]));")
   
   scode <- make_stancode(
-    bf(count ~ Trt_c, hu ~ Trt_c), epilepsy, 
+    bf(count ~ Trt, hu ~ Trt), epilepsy, 
     family = hurdle_gamma(link_hu = "identity")
   )
   expect_match2(scode, "target += hurdle_gamma_lpdf(Y[n] | shape, mu[n], hu[n])")
@@ -1194,7 +1250,7 @@ test_that("Stan code of mixture model is correct", {
                          data = data, mixture(Gamma("log"), weibull))
   expect_match(scode, "data \\{[^\\}]*real<lower=0,upper=1> theta1;")
   expect_match(scode, "data \\{[^\\}]*real<lower=0,upper=1> theta2;")
-  expect_match(scode, "shape1\\[n\\] = exp\\(shape1\\[n\\]\\); \\\n    mu1\\[n\\] = ")
+  expect_match(scode, "shape1\\[n\\] = exp\\(shape1\\[n\\]\\);\\\n    mu1\\[n\\] = ")
   expect_match2(scode, "ps[1] = log(theta1) + gamma_lpdf(Y[n] | shape1[n], mu1[n]);")
   expect_match2(scode, "target += weights[n] * log_sum_exp(ps);")
   
@@ -1292,6 +1348,7 @@ test_that("sparse matrix multiplication is applied correctly", {
 })
 
 test_that("Stan code for Gaussian processes is correct", {
+  set.seed(1234)
   dat <- data.frame(y = rnorm(40), x1 = rnorm(40), x2 = rnorm(40),
                     z = factor(rep(3:6, each = 10)))
   
@@ -1451,7 +1508,7 @@ test_that("Stan code for missing value terms works correctly", {
   bform <- bf(y ~ mi(x) + (mi(x) | g)) + bf(x | mi() ~ 1) + set_rescor(FALSE)
   scode <- make_stancode(bform, dat)
   expect_match2(scode, 
-    "(bsp_y[1] + r_1_y_2[J_1[n]]) * Yl_x[n] + r_1_y_1[J_1[n]] * Z_1_y_1[n];"
+    "(bsp_y[1] + r_1_y_2[J_1_y[n]]) * Yl_x[n] + r_1_y_1[J_1_y[n]] * Z_1_y_1[n];"
   )
   
   bform <- bf(y ~ a, a ~ mi(x), nl = TRUE) + bf(x | mi() ~ 1) + set_rescor(FALSE)
@@ -1494,7 +1551,22 @@ test_that("Stan code for overimputation works correctly", {
   expect_match2(scode, 
     "target += normal_lpdf(Y_xx[Jme_xx] | Yl_xx[Jme_xx], noise_xx[Jme_xx])"
   )
-  expect_match2(scode, "vector[N] Yl_xx;")
+  expect_match2(scode, "vector[N_xx] Yl_xx;")
+})
+
+test_that("Stan code for advanced count data distribution is correct", {
+  scode <- make_stancode(
+    count ~ zAge + zBase * Trt + (1|patient),
+    data = epilepsy, family = brmsfamily("discrete_weibull")
+  )
+  expect_match2(scode, "mu[n] = inv_logit(mu[n]);")
+  expect_match2(scode, "target += discrete_weibull_lpmf(Y[n] | mu[n], shape);")
+  
+  scode <- make_stancode(
+    count ~ zAge + zBase * Trt + (1|patient),
+    data = epilepsy, family = brmsfamily("com_poisson")
+  )
+  expect_match2(scode, "target += com_poisson_log_lpmf(Y[n] | mu[n], shape);")
 })
 
 test_that("argument 'stanvars' is handled correctly", {
@@ -1509,7 +1581,7 @@ test_that("argument 'stanvars' is handled correctly", {
   bprior <- prior(multi_normal(M, V), class = "b")
   stanvars <- stanvar(rep(0, 2), "M", scode = "  vector[K] M;") +
     stanvar(diag(2), "V", scode = "  matrix[K, K] V;") 
-  scode <- make_stancode(count ~ Trt + log_Base4_c, epilepsy,
+  scode <- make_stancode(count ~ Trt + zBase, epilepsy,
                          prior = bprior, stanvars = stanvars)
   expect_match2(scode, "vector[K] M;")
   expect_match2(scode, "matrix[K, K] V;")
@@ -1519,7 +1591,7 @@ test_that("argument 'stanvars' is handled correctly", {
     set_prior("target += normal_lpdf(tau | 0, 10)", check = FALSE)
   stanvars <- stanvar(scode = "real<lower=0> tau;", 
                       block = "parameters")
-  scode <- make_stancode(count ~ Trt + log_Base4_c, epilepsy,
+  scode <- make_stancode(count ~ Trt + zBase, epilepsy,
                          prior = bprior, stanvars = stanvars)
   expect_match2(scode, "real<lower=0> tau;")
   expect_match2(scode, "target += normal_lpdf(b | 0, tau);")
@@ -1538,7 +1610,7 @@ test_that("argument 'stanvars' is handled correctly", {
   stanvars <- stanvar(scode = "vector[Ksp] zbsp;", block = "parameters") +
     stanvar(scode = "real<lower=0> tau;", block = "parameters") +
     stanvar(scode = "vector[Ksp] bsp = zbsp * tau;", 
-            block="tparameters", name = "bsp")
+            block = "tparameters", name = "bsp")
   scode <- make_stancode(count ~ mo(Base), epilepsy, stanvars = stanvars)
   expect_match2(scode, "vector[Ksp] bsp = zbsp * tau;")
 })
@@ -1602,7 +1674,7 @@ test_that("custom families are handled correctly", {
     bf(y ~ x, tau ~ x), data = dat, family = beta_binomial2, 
     stanvars = stanvars
   )
-  expect_match2(scode, "tau[n] = exp(tau[n]); ")
+  expect_match2(scode, "tau[n] = exp(tau[n]);")
   expect_match2(scode, "target += beta_binomial2_lpmf(Y[n] | mu[n], tau[n], trials[n]);")
   
   # check custom families in mixture models
@@ -1626,7 +1698,8 @@ test_that("likelihood of distributional beta models is correct", {
 
 test_that("student-t group-level effects work without errors", {
   scode <- make_stancode(count ~ Trt + (1|gr(patient, dist = "st")), epilepsy)
-  expect_match2(scode, "sqrt(df_1 * udf_1) * sd_1[1] * (z_1[1]);")
+  expect_match2(scode, "vector[N_1] dfm_1 = sqrt(df_1 * udf_1);")
+  expect_match2(scode, "dfm_1 .* (sd_1[1] * (z_1[1]));")
   expect_match2(scode, "target += gamma_lpdf(df_1 | 2, 0.1);")
   expect_match2(scode, "target += inv_chi_square_lpdf(udf_1 | df_1);")
   
@@ -1635,6 +1708,22 @@ test_that("student-t group-level effects work without errors", {
     count ~ Trt + (Trt|gr(patient, dist = "st")), 
     epilepsy, prior = bprior
   )
-  expect_match2(scode, "sqrt(df_1 * udf_1) * (diag_pre_multiply(sd_1, L_1) * z_1)';")
+  expect_match2(scode,
+    "rep_matrix(dfm_1, M_1) .* (diag_pre_multiply(sd_1, L_1) * z_1)';"
+  )
   expect_match2(scode, "target += normal_lpdf(df_1 | 20, 5);")
 })
+
+test_that("centering design matrices can be changed correctly", {
+  dat <- data.frame(y = 1:10, x = 1:10)
+  scode <- make_stancode(
+    bf(y ~ x, center = FALSE), data = dat,
+    prior = prior(normal(0,1), coef = Intercept)
+  )
+  expect_match2(scode, "mu = X * b;")
+  expect_match2(scode, "target += normal_lpdf(b[1] | 0, 1);")
+  
+  bform <- bf(y ~ eta, nl = TRUE) + lf(eta ~ x, center = TRUE)
+  scode <- make_stancode(bform, data = dat)
+  expect_match2(scode, "nlp_eta = temp_eta_Intercept + Xc_eta * b_eta;")
+})
